@@ -18,6 +18,7 @@
 package org.apache.hadoop.hbase.zookeeper;
 
 import java.io.IOException;
+import java.util.List;
 import java.util.Map.Entry;
 import java.util.Properties;
 import org.apache.hadoop.conf.Configuration;
@@ -25,6 +26,8 @@ import org.apache.hadoop.hbase.HConstants;
 import org.apache.hadoop.util.StringUtils;
 import org.apache.yetus.audience.InterfaceAudience;
 
+import org.apache.hbase.thirdparty.com.google.common.base.Splitter;
+
 /**
  * Utility methods for reading, and building the ZooKeeper configuration. The order and priority for
  * reading the config are as follows: (1). Property with "hbase.zookeeper.property." prefix from
@@ -34,7 +37,6 @@ import org.apache.yetus.audience.InterfaceAudience;
 public final class ZKConfig {
 
   private static final String VARIABLE_START = "${";
-  private static final String ZOOKEEPER_JAVA_PROPERTY_PREFIX = "zookeeper.";
 
   private ZKConfig() {
   }
@@ -50,12 +52,16 @@ public final class ZKConfig {
   }
 
   /**
-   * Directly map all the hbase.zookeeper.property.KEY properties. Synchronize on conf so no loading
-   * of configs while we iterate
+   * Make a Properties object holding ZooKeeper config. Parses the corresponding config options from
+   * the HBase XML configs and generates the appropriate ZooKeeper properties.
+   * @param conf Configuration to read from.
+   * @return Properties holding mappings representing ZooKeeper config file.
    */
-  private static Properties extractZKPropsFromHBaseConfig(final Configuration conf) {
+  private static Properties makeZKPropsFromHbaseConfig(Configuration conf) {
     Properties zkProperties = new Properties();
 
+    // Directly map all of the hbase.zookeeper.property.KEY properties.
+    // Synchronize on conf so no loading of configs while we iterate
     synchronized (conf) {
       for (Entry<String, String> entry : conf) {
         String key = entry.getKey();
@@ -71,18 +77,6 @@ public final class ZKConfig {
       }
     }
 
-    return zkProperties;
-  }
-
-  /**
-   * Make a Properties object holding ZooKeeper config. Parses the corresponding config options from
-   * the HBase XML configs and generates the appropriate ZooKeeper properties.
-   * @param conf Configuration to read from.
-   * @return Properties holding mappings representing ZooKeeper config file.
-   */
-  private static Properties makeZKPropsFromHbaseConfig(Configuration conf) {
-    Properties zkProperties = extractZKPropsFromHBaseConfig(conf);
-
     // If clientPort is not set, assign the default.
     if (zkProperties.getProperty(HConstants.CLIENT_PORT_STR) == null) {
       zkProperties.put(HConstants.CLIENT_PORT_STR, HConstants.DEFAULT_ZOOKEEPER_CLIENT_PORT);
@@ -128,7 +122,6 @@ public final class ZKConfig {
    * @return Quorum servers
    */
   public static String getZKQuorumServersString(Configuration conf) {
-    setZooKeeperClientSystemProperties(HConstants.ZK_CFG_PROPERTY_PREFIX, conf);
     return getZKQuorumServersStringFromHbaseConfig(conf);
   }
 
@@ -179,27 +172,28 @@ public final class ZKConfig {
    * @return the three configuration in the described order
    */
   public static ZKClusterKey transformClusterKey(String key) throws IOException {
-    String[] parts = key.split(":");
+    List<String> parts = Splitter.on(':').splitToList(key);
+    String[] partsArray = parts.toArray(new String[parts.size()]);
 
-    if (parts.length == 3) {
-      if (!parts[2].matches("/.*[^/]")) {
+    if (partsArray.length == 3) {
+      if (!partsArray[2].matches("/.*[^/]")) {
         throw new IOException("Cluster key passed " + key + " is invalid, the format should be:"
           + HConstants.ZOOKEEPER_QUORUM + ":" + HConstants.ZOOKEEPER_CLIENT_PORT + ":"
           + HConstants.ZOOKEEPER_ZNODE_PARENT);
       }
-      return new ZKClusterKey(parts[0], Integer.parseInt(parts[1]), parts[2]);
+      return new ZKClusterKey(partsArray[0], Integer.parseInt(partsArray[1]), partsArray[2]);
     }
 
-    if (parts.length > 3) {
+    if (partsArray.length > 3) {
       // The quorum could contain client port in server:clientport format, try to transform more.
-      String zNodeParent = parts[parts.length - 1];
+      String zNodeParent = partsArray[partsArray.length - 1];
       if (!zNodeParent.matches("/.*[^/]")) {
         throw new IOException("Cluster key passed " + key + " is invalid, the format should be:"
           + HConstants.ZOOKEEPER_QUORUM + ":" + HConstants.ZOOKEEPER_CLIENT_PORT + ":"
           + HConstants.ZOOKEEPER_ZNODE_PARENT);
       }
 
-      String clientPort = parts[parts.length - 2];
+      String clientPort = partsArray[partsArray.length - 2];
 
       // The first part length is the total length minus the lengths of other parts and minus 2 ":"
       int endQuorumIndex = key.length() - zNodeParent.length() - clientPort.length() - 2;
@@ -209,7 +203,7 @@ public final class ZKConfig {
       // The common case is that every server has its own client port specified - this means
       // that (total parts - the ZNodeParent part - the ClientPort part) is equal to
       // (the number of "," + 1) - "+ 1" because the last server has no ",".
-      if ((parts.length - 2) == (serverHosts.length + 1)) {
+      if ((partsArray.length - 2) == (serverHosts.length + 1)) {
         return new ZKClusterKey(quorumStringInput, Integer.parseInt(clientPort), zNodeParent);
       }
 
@@ -301,7 +295,6 @@ public final class ZKConfig {
    * @return Client quorum servers, or null if not specified
    */
   public static String getClientZKQuorumServersString(Configuration conf) {
-    setZooKeeperClientSystemProperties(HConstants.ZK_CFG_PROPERTY_PREFIX, conf);
     String clientQuromServers = conf.get(HConstants.CLIENT_ZOOKEEPER_QUORUM);
     if (clientQuromServers == null) {
       return null;
@@ -314,15 +307,4 @@ public final class ZKConfig {
     final String[] serverHosts = StringUtils.getStrings(clientQuromServers);
     return buildZKQuorumServerString(serverHosts, clientZkClientPort);
   }
-
-  private static void setZooKeeperClientSystemProperties(String prefix, Configuration conf) {
-    Properties zkProperties = extractZKPropsFromHBaseConfig(conf);
-    for (Entry<Object, Object> entry : zkProperties.entrySet()) {
-      String key = entry.getKey().toString().trim();
-      String value = entry.getValue().toString().trim();
-      if (System.getProperty(ZOOKEEPER_JAVA_PROPERTY_PREFIX + key) == null) {
-        System.setProperty(ZOOKEEPER_JAVA_PROPERTY_PREFIX + key, value);
-      }
-    }
-  }
 }
