@@ -89,6 +89,8 @@ public class MetricsRegionServerSourceImpl extends BaseSourceImpl
   private final MetricHistogram pausesWithGc;
   private final MetricHistogram pausesWithoutGc;
 
+  private final MutableFastCounter scannerLeaseExpiredCount;
+
   public MetricsRegionServerSourceImpl(MetricsRegionServerWrapper rsWrap) {
     this(METRICS_NAME, METRICS_DESCRIPTION, METRICS_CONTEXT, METRICS_JMX_CONTEXT, rsWrap);
   }
@@ -174,6 +176,9 @@ public class MetricsRegionServerSourceImpl extends BaseSourceImpl
       getMetricsRegistry().newCounter(WARN_THRESHOLD_COUNT_KEY, WARN_THRESHOLD_COUNT_DESC, 0L);
     pausesWithGc = getMetricsRegistry().newTimeHistogram(PAUSE_TIME_WITH_GC_KEY);
     pausesWithoutGc = getMetricsRegistry().newTimeHistogram(PAUSE_TIME_WITHOUT_GC_KEY);
+
+    scannerLeaseExpiredCount = getMetricsRegistry().newCounter(SCANNER_LEASE_EXPIRED_COUNT,
+      SCANNER_LEASE_EXPIRED_COUNT_DESC, 0L);
   }
 
   @Override
@@ -317,6 +322,11 @@ public class MetricsRegionServerSourceImpl extends BaseSourceImpl
     }
   }
 
+  @Override
+  public void incrScannerLeaseExpired() {
+    scannerLeaseExpiredCount.incr();
+  }
+
   /**
    * Yes this is a get function that doesn't return anything. Thanks Hadoop for breaking all
    * expectations of java programmers. Instead of returning anything Hadoop metrics expects
@@ -497,6 +507,13 @@ public class MetricsRegionServerSourceImpl extends BaseSourceImpl
         rsWrap.getTotalStaticIndexSize())
       .addGauge(Interns.info(STATIC_BLOOM_SIZE, STATIC_BLOOM_SIZE_DESC),
         rsWrap.getTotalStaticBloomSize())
+      .addCounter(Interns.info(BLOOM_FILTER_REQUESTS_COUNT, BLOOM_FILTER_REQUESTS_COUNT_DESC),
+        rsWrap.getBloomFilterRequestsCount())
+      .addCounter(
+        Interns.info(BLOOM_FILTER_NEGATIVE_RESULTS_COUNT, BLOOM_FILTER_NEGATIVE_RESULTS_COUNT_DESC),
+        rsWrap.getBloomFilterNegativeResultsCount())
+      .addCounter(Interns.info(BLOOM_FILTER_ELIGIBLE_REQUESTS_COUNT,
+        BLOOM_FILTER_ELIGIBLE_REQUESTS_COUNT_DESC), rsWrap.getBloomFilterEligibleRequestsCount())
       .addGauge(Interns.info(NUMBER_OF_MUTATIONS_WITHOUT_WAL, NUMBER_OF_MUTATIONS_WITHOUT_WAL_DESC),
         rsWrap.getNumMutationsWithoutWAL())
       .addGauge(Interns.info(DATA_SIZE_WITHOUT_WAL, DATA_SIZE_WITHOUT_WAL_DESC),
@@ -584,10 +601,9 @@ public class MetricsRegionServerSourceImpl extends BaseSourceImpl
         Interns.info(BYTE_BUFF_ALLOCATOR_TOTAL_BUFFER_COUNT,
           BYTE_BUFF_ALLOCATOR_TOTAL_BUFFER_COUNT_DESC),
         rsWrap.getByteBuffAllocatorTotalBufferCount())
-      .addGauge(
-        Interns.info(BYTE_BUFF_ALLOCATOR_USED_BUFFER_COUNT,
-          BYTE_BUFF_ALLOCATOR_USED_BUFFER_COUNT_DESC),
-        rsWrap.getByteBuffAllocatorUsedBufferCount());
+      .addGauge(Interns.info(BYTE_BUFF_ALLOCATOR_USED_BUFFER_COUNT,
+        BYTE_BUFF_ALLOCATOR_USED_BUFFER_COUNT_DESC), rsWrap.getByteBuffAllocatorUsedBufferCount())
+      .addGauge(Interns.info(ACTIVE_SCANNERS, ACTIVE_SCANNERS_DESC), rsWrap.getActiveScanners());
   }
 
   @Override
