@@ -26,10 +26,6 @@ import static org.junit.Assert.fail;
 import java.io.IOException;
 import java.util.Arrays;
 import java.util.List;
-import java.util.concurrent.ExecutionException;
-import java.util.concurrent.TimeUnit;
-import java.util.stream.Collectors;
-import java.util.stream.IntStream;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.CallQueueTooBigException;
 import org.apache.hadoop.hbase.HBaseClassTestRule;
@@ -51,12 +47,10 @@ import org.apache.hadoop.hbase.testclassification.ClientTests;
 import org.apache.hadoop.hbase.testclassification.MediumTests;
 import org.apache.hadoop.hbase.util.Bytes;
 import org.junit.AfterClass;
-import org.junit.Assert;
 import org.junit.BeforeClass;
 import org.junit.ClassRule;
 import org.junit.Test;
 import org.junit.experimental.categories.Category;
-import org.junit.function.ThrowingRunnable;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -108,128 +102,6 @@ public class TestMetaCache {
     TEST_UTIL.shutdownMiniCluster();
   }
 
-  @Test
-  public void testMergeEmptyWithMetaCache() throws Throwable {
-    TableName tableName = TableName.valueOf("MergeEmpty");
-    byte[] family = Bytes.toBytes("CF");
-    TableDescriptor td = TableDescriptorBuilder.newBuilder(tableName)
-      .setColumnFamily(ColumnFamilyDescriptorBuilder.of(family)).build();
-    TEST_UTIL.getAdmin().createTable(td, new byte[][] { Bytes.toBytes(2), Bytes.toBytes(5) });
-    TEST_UTIL.waitTableAvailable(tableName);
-    TEST_UTIL.waitUntilNoRegionsInTransition();
-    RegionInfo regionA = null;
-    RegionInfo regionB = null;
-    RegionInfo regionC = null;
-    for (RegionInfo region : TEST_UTIL.getAdmin().getRegions(tableName)) {
-      if (region.getStartKey().length == 0) {
-        regionA = region;
-      } else if (Bytes.equals(region.getStartKey(), Bytes.toBytes(2))) {
-        regionB = region;
-      } else if (Bytes.equals(region.getStartKey(), Bytes.toBytes(5))) {
-        regionC = region;
-      }
-    }
-
-    assertNotNull(regionA);
-    assertNotNull(regionB);
-    assertNotNull(regionC);
-
-    TEST_UTIL.getConfiguration().setBoolean(MetricsConnection.CLIENT_SIDE_METRICS_ENABLED_KEY,
-      true);
-    try (Connection conn = ConnectionFactory.createConnection(TEST_UTIL.getConfiguration());
-      AsyncConnection asyncConn =
-        ConnectionFactory.createAsyncConnection(TEST_UTIL.getConfiguration()).get()) {
-      ConnectionImplementation connImpl = (ConnectionImplementation) conn;
-      AsyncConnectionImpl asyncConnImpl = (AsyncConnectionImpl) asyncConn;
-
-      MetricsConnection metrics = connImpl.getConnectionMetrics();
-      MetricsConnection asyncMetrics = asyncConnImpl.getConnectionMetrics().get();
-
-      // warm meta cache
-      conn.getRegionLocator(tableName).getAllRegionLocations();
-      asyncConn.getRegionLocator(tableName).getAllRegionLocations().get();
-
-      Assert.assertEquals(3, TEST_UTIL.getAdmin().getRegions(tableName).size());
-
-      // Merge the 3 regions into one
-      TEST_UTIL.getAdmin().mergeRegionsAsync(
-        new byte[][] { regionA.getRegionName(), regionB.getRegionName(), regionC.getRegionName() },
-        false).get(30, TimeUnit.SECONDS);
-
-      Assert.assertEquals(1, TEST_UTIL.getAdmin().getRegions(tableName).size());
-
-      Table table = conn.getTable(tableName);
-      AsyncTable<?> asyncTable = asyncConn.getTable(tableName);
-
-      // This request should cause us to cache the newly merged region.
-      // As part of caching that region, it should clear out any cached merge parent regions which
-      // are overlapped by the new region. That way, subsequent calls below won't fall into the
-      // bug in HBASE-27650. Otherwise, a request for row 6 would always get stuck on cached
-      // regionB and we'd continue to see cache misses below.
-      assertTrue(executeAndGetNewMisses(() -> table.get(new Get(Bytes.toBytes(6))), metrics) > 0);
-      assertTrue(
-        executeAndGetNewMisses(() -> asyncTable.get(new Get(Bytes.toBytes(6))).get(), asyncMetrics)
-            > 0);
-
-      // We verify no new cache misses here due to above, which proves we've fixed up the cache
-      assertEquals(0, executeAndGetNewMisses(() -> table.get(new Get(Bytes.toBytes(6))), metrics));
-      assertEquals(0, executeAndGetNewMisses(() -> asyncTable.get(new Get(Bytes.toBytes(6))).get(),
-        asyncMetrics));
-    }
-  }
-
-  private long executeAndGetNewMisses(ThrowingRunnable runnable, MetricsConnection metrics)
-    throws Throwable {
-    long lastVal = metrics.getMetaCacheMisses();
-    runnable.run();
-    long curVal = metrics.getMetaCacheMisses();
-    return curVal - lastVal;
-  }
-
-  /**
-   * Test that our cleanOverlappingRegions doesn't incorrectly remove regions from cache. Originally
-   * encountered when using floorEntry rather than lowerEntry.
-   */
-  @Test
-  public void testAddToCacheReverse() throws IOException, InterruptedException, ExecutionException {
-    try (
-      AsyncConnectionImpl asyncConn = (AsyncConnectionImpl) ConnectionFactory
-        .createAsyncConnection(TEST_UTIL.getConfiguration()).get();
-      ConnectionImplementation conn = (ConnectionImplementation) ConnectionFactory
-        .createConnection(TEST_UTIL.getConfiguration())) {
-
-      AsyncNonMetaRegionLocator asyncLocator = asyncConn.getLocator().getNonMetaRegionLocator();
-
-      TableName tableName = TableName.valueOf("testAddToCache");
-      byte[] family = Bytes.toBytes("CF");
-      TableDescriptor td = TableDescriptorBuilder.newBuilder(tableName)
-        .setColumnFamily(ColumnFamilyDescriptorBuilder.of(family)).build();
-      int maxSplits = 10;
-      List<byte[]> splits =
-        IntStream.range(1, maxSplits).mapToObj(Bytes::toBytes).collect(Collectors.toList());
-
-      TEST_UTIL.getAdmin().createTable(td, splits.toArray(new byte[0][]));
-      TEST_UTIL.waitTableAvailable(tableName);
-      TEST_UTIL.waitUntilNoRegionsInTransition();
-      conn.getRegionLocator(tableName);
-
-      assertEquals(splits.size() + 1, TEST_UTIL.getAdmin().getRegions(tableName).size());
-
-      RegionLocator locatorForTable = conn.getRegionLocator(tableName);
-      AsyncTableRegionLocator asyncLocatorForTable = asyncConn.getRegionLocator(tableName);
-      for (int i = maxSplits; i >= 0; i--) {
-        locatorForTable.getRegionLocation(Bytes.toBytes(i));
-        asyncLocatorForTable.getRegionLocation(Bytes.toBytes(i));
-      }
-
-      for (int i = 0; i < maxSplits; i++) {
-        assertNotNull(asyncLocator.getRegionLocationInCache(tableName, Bytes.toBytes(i)));
-        assertNotNull(conn.getCachedLocation(tableName, Bytes.toBytes(i)));
-      }
-    }
-
-  }
-
   @Test
   public void testPreserveMetaCacheOnException() throws Exception {
     ((FakeRSRpcServices) badRS.getRSRpcServices())
@@ -371,8 +243,8 @@ public class TestMetaCache {
 
       // obtain the client metrics
       MetricsConnection metrics = conn.getConnectionMetrics();
-      long preGetRegionClears = metrics.metaCacheNumClearRegion.getCount();
-      long preGetServerClears = metrics.metaCacheNumClearServer.getCount();
+      long preGetRegionClears = metrics.getMetaCacheNumClearRegion().getCount();
+      long preGetServerClears = metrics.getMetaCacheNumClearServer().getCount();
 
       // attempt a get on the test table
       Get get = new Get(row);
@@ -384,8 +256,8 @@ public class TestMetaCache {
       }
 
       // verify that no cache clearing took place
-      long postGetRegionClears = metrics.metaCacheNumClearRegion.getCount();
-      long postGetServerClears = metrics.metaCacheNumClearServer.getCount();
+      long postGetRegionClears = metrics.getMetaCacheNumClearRegion().getCount();
+      long postGetServerClears = metrics.getMetaCacheNumClearServer().getCount();
       assertEquals(preGetRegionClears, postGetRegionClears);
       assertEquals(preGetServerClears, postGetServerClears);
     } finally {
@@ -563,7 +435,6 @@ public class TestMetaCache {
     conf.setInt(HConstants.HBASE_CLIENT_RETRIES_NUMBER, 0);
     conf.setLong(HConstants.HBASE_CLIENT_META_OPERATION_TIMEOUT, 2000);
     conf.setLong(HConstants.HBASE_CLIENT_SCANNER_TIMEOUT_PERIOD, 2000);
-    conf.setBoolean(MetricsConnection.CLIENT_SIDE_METRICS_ENABLED_KEY, true);
 
     try (ConnectionImplementation conn =
       (ConnectionImplementation) ConnectionFactory.createConnection(conf)) {
@@ -588,28 +459,6 @@ public class TestMetaCache {
 
       assertTrue(client1.getException() instanceof LockTimeoutException
         ^ client2.getException() instanceof LockTimeoutException);
-
-      // obtain the client metrics
-      MetricsConnection metrics = conn.getConnectionMetrics();
-      long queueCount = metrics.userRegionLockQueueHist.getCount();
-      assertEquals("Queue of userRegionLock should be updated twice. queueCount: " + queueCount, 2,
-        queueCount);
-
-      long timeoutCount = metrics.userRegionLockTimeoutCount.getCount();
-      assertEquals("Timeout of userRegionLock should happen once. timeoutCount: " + timeoutCount, 1,
-        timeoutCount);
-
-      long waitingTimerCount = metrics.userRegionLockWaitingTimer.getCount();
-      assertEquals("userRegionLock should be grabbed successfully once. waitingTimerCount: "
-        + waitingTimerCount, 1, waitingTimerCount);
-
-      long heldTimerCount = metrics.userRegionLockHeldTimer.getCount();
-      assertEquals(
-        "userRegionLock should be held successfully once. heldTimerCount: " + heldTimerCount, 1,
-        heldTimerCount);
-      double heldTime = metrics.userRegionLockHeldTimer.getSnapshot().getMax();
-      assertTrue("Max held time should be greater than 2 seconds. heldTime: " + heldTime,
-        heldTime >= 2E9);
     }
   }
 
